<?php

namespace App\Http\Controllers;

use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use Illuminate\Contracts\Routing\ResponseFactory;
use Illuminate\Support\Facades\Log;

use App\Http\Controllers\AuthController;
use App\Http\Controllers\UserController;
use App\Http\Controllers\PermissionController;

class SiteController extends Controller {
  private $objAuth;
  private $objUser;
  private $objPermission;

  public function __construct() {
    $this->objAuth = new AuthController();
    $this->objUser = new UserController();
    $this->objPermission = new PermissionController();
  }

  // Posts
  public function getPosts2 () {
    $kero_token = (isset($_COOKIE['kero_token']) ? $_COOKIE['kero_token'] : '');
    $check = $this->objAuth->checkLegit($kero_token);

    // Load group colours.
    $ucol = $this->objUser->getGroupColours();
    $valid = $this->objAuth->getPermissions($kero_token);

    // Load content.
    if ($valid['blg_editpost']) {
      $get = DB::table('blg_content')
        ->join('users', 'blg_content.user_id', '=', 'users.id')
        ->join('usr_details', 'usr_details.user_id', '=', 'blg_content.user_id')
        ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_content.user_id')
        ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_content.user_id')
        ->where('isPost', 1)
        ->orderBy('publish_date', 'desc')
        ->get(array(
          'blg_content.id',
          'blg_content.user_id',
          'title',
          'slug',
          'post_date',
          'publish_date',
          'public_status',
          'message',
          'username',
          'perm_id',
          'gender',
          'avatar',
          'name_style',
          'display_name'
        ));
    }
    else {
      $get = DB::table('blg_content')
        ->join('users', 'blg_content.user_id', '=', 'users.id')
        ->join('usr_details', 'usr_details.user_id', '=', 'blg_content.user_id')
        ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_content.user_id')
        ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_content.user_id')
        ->where('public_status', 0)
        ->where('isPost', 1)
        ->orderBy('publish_date', 'desc')
        ->get(array(
          'blg_content.id',
          'blg_content.user_id',
          'title',
          'slug',
          'post_date',
          'publish_date',
          'public_status',
          'message',
          'username',
          'perm_id',
          'gender',
          'avatar',
          'name_style',
          'display_name'
        ));
    }

    $res = array();
    $key = 0;
    setlocale(LC_ALL, 'ja_JP.utf8');

    foreach ($get as $i) {
      $showName = '';
      $showCol = '';
      $showGroupName = '';

      // Display name or username?
      if (!empty($i->display_name)) {
        $showName = $i->display_name;
      }
      else {
        $showName = $i->username;
      }

      // Custom name styling or default?
      if (!empty($i->name_style)) {
        $showCol = $i->name_style;
      }
      else {
        foreach ($ucol as $j) {
          if ($j->id == $i->perm_id) {
            if ($i->gender == 1) $showCol = $j->colour_m;
            else if ($i->gender == 2) $showCol = $j->colour_f;
            else $showCol = $j->colour_u;
          }
        }
      }

      // Group names.
      $gname = $this->objUser->getGroupName($i->user_id);
      $showGroupName = $gname[0]->name;

      array_push($res, [
        'key' => $key,
        'id' => $i->id,
        'user_id' => $i->user_id,
        'title' => $i->title,
        'slug' => $i->slug,
        'post_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->post_date),
        'publish_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->publish_date),
        'public_status' => $i->public_status,
        'message' => $i->message,
        'avatar' => $i->avatar,
        'showcol' => $showCol,
        'showname' => $showName
      ]);
      $key++;
    }

    return $res;
  }

  public function getPosts(Request $r) { // /api/rpc/site/post/get/all
    $check = $this->objAuth->checkLegit($r->kero_token);

    // Load group colours.
    $ucol = $this->objUser->getGroupColours();
    $valid = $this->objAuth->getPermissions($r->kero_token);

    // Load content.
    if ($valid['blg_editpost']) {
      $get = DB::table('blg_content')
        ->join('users', 'blg_content.user_id', '=', 'users.id')
        ->join('usr_details', 'usr_details.user_id', '=', 'blg_content.user_id')
        ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_content.user_id')
        ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_content.user_id')
        ->where('isPost', 1)
        ->orderBy('publish_date', 'desc')
        ->get(array(
          'blg_content.id',
          'blg_content.user_id',
          'title',
          'slug',
          'post_date',
          'publish_date',
          'public_status',
          'message',
          'username',
          'perm_id',
          'gender',
          'avatar',
          'name_style',
          'display_name'
        ));
    }
    else {
      $get = DB::table('blg_content')
        ->join('users', 'blg_content.user_id', '=', 'users.id')
        ->join('usr_details', 'usr_details.user_id', '=', 'blg_content.user_id')
        ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_content.user_id')
        ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_content.user_id')
        ->where('public_status', 0)
        ->where('isPost', 1)
        ->orderBy('publish_date', 'desc')
        ->get(array(
          'blg_content.id',
          'blg_content.user_id',
          'title',
          'slug',
          'post_date',
          'publish_date',
          'public_status',
          'message',
          'username',
          'perm_id',
          'gender',
          'avatar',
          'name_style',
          'display_name'
        ));
    }

    $res = array();
    $key = 0;
    setlocale(LC_ALL, 'ja_JP.utf8');

    foreach ($get as $i) {
      $showName = '';
      $showCol = '';
      $showGroupName = '';

      // Display name or username?
      if (!empty($i->display_name)) {
        $showName = $i->display_name;
      }
      else {
        $showName = $i->username;
      }

      // Custom name styling or default?
      if (!empty($i->name_style)) {
        $showCol = $i->name_style;
      }
      else {
        foreach ($ucol as $j) {
          if ($j->id == $i->perm_id) {
            if ($i->gender == 1) $showCol = $j->colour_m;
            else if ($i->gender == 2) $showCol = $j->colour_f;
            else $showCol = $j->colour_u;
          }
        }
      }

      // Group names.
      $gname = $this->objUser->getGroupName($i->user_id);
      $showGroupName = $gname[0]->name;

      array_push($res, [
        'key' => $key,
        'id' => $i->id,
        'user_id' => $i->user_id,
        'title' => $i->title,
        'slug' => $i->slug,
        'post_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->post_date),
        'publish_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->publish_date),
        'public_status' => $i->public_status,
        'message' => $i->message,
        'avatar' => $i->avatar,
        'showcol' => $showCol,
        'showname' => $showName
      ]);
      $key++;
    }

    return $res;
  }

  public function getUserPosts($id) { // /api/rpc/site/post/get/user/id
    $get = DB::table('blg_content')
      ->select('id', 'title', 'slug', 'post_date', 'publish_date', 'public_status', 'message')
      ->where('public_status', 0)
      ->where('isPost', 1)
      ->where('user_id', $id)
      ->orderBy('publish_date', 'desc')
      ->get();

    $res = array();
    $key = 0;

    setlocale(LC_ALL, 'ja_JP.utf8');

    foreach ($get as $i) {
      array_push($res, [
        'key' => $key,
        'id' => $i->id,
        'title' => $i->title,
        'slug' => $i->slug,
        'post_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->post_date),
        'publish_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->publish_date),
        'public_status' => $i->public_status,
        'message' => $i->message
      ]);
      $key++;
    }

    return $res;
  }

  public function getPostSlug($id) { // /api/rpc/site/post/get/id/id
    $get = DB::table('blg_content')
      ->select('slug')
      ->where('public_status', 0)
      ->where('isPost', 1)
      ->where('id', $id)
      ->get();

    $res = array();

    foreach ($get as $i) {
      array_push($res, [
        'slug' => $i->slug
      ]);
    }

    return $res;
  }

  public function getPost2 ($slug, $kero) {
    $check = $this->objAuth->checkLegit($kero);
    $valid = $this->objAuth->getPermissions($kero);

    // Load group colours.
    $ucol = $this->objUser->getGroupColours();

    // Load content.
    if ($valid['blg_editpost']) {
      $get = DB::table('blg_content')
        ->join('users', 'blg_content.user_id', '=', 'users.id')
        ->join('usr_details', 'usr_details.user_id', '=', 'blg_content.user_id')
        ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_content.user_id')
        ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_content.user_id')
        ->where('isPost', 1)
        ->where('slug', $slug)
        ->get(array(
          'blg_content.id',
          'blg_content.user_id',
          'title',
          'slug',
          'post_date',
          'publish_date',
          'public_status',
          'message',
          'username',
          'perm_id',
          'gender',
          'avatar',
          'name_style',
          'display_name'
        ));
    }
    else {
      $get = DB::table('blg_content')
        ->join('users', 'blg_content.user_id', '=', 'users.id')
        ->join('usr_details', 'usr_details.user_id', '=', 'blg_content.user_id')
        ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_content.user_id')
        ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_content.user_id')
        ->where('public_status', 0)
        ->where('isPost', 1)
        ->where('slug', $slug)
        ->get(array(
          'blg_content.id',
          'blg_content.user_id',
          'title',
          'slug',
          'post_date',
          'publish_date',
          'public_status',
          'message',
          'username',
          'perm_id',
          'gender',
          'avatar',
          'name_style',
          'display_name'
        ));
    }

    $res = array();
    setlocale(LC_ALL, 'ja_JP.utf8');

    foreach ($get as $i) {
      $showName = '';
      $showCol = '';
      $showGroupName = '';

      // Display name or username?
      if (!empty($i->display_name)) {
        $showName = $i->display_name;
      }
      else {
        $showName = $i->username;
      }

      // Custom name styling or default?
      if (!empty($i->name_style)) {
        $showCol = $i->name_style;
      }
      else {
        foreach ($ucol as $j) {
          if ($j->id == $i->perm_id) {
            if ($i->gender == 1) $showCol = $j->colour_m;
            else if ($i->gender == 2) $showCol = $j->colour_f;
            else $showCol = $j->colour_u;
          }
        }
      }

      // Group names.
      $gname = $this->objUser->getGroupName($i->user_id);
      $showGroupName = $gname[0]->name;

      array_push($res, [
        'id' => $i->id,
        'user_id' => $i->user_id,
        'title' => $i->title,
        'slug' => $i->slug,
        'post_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->post_date),
        'publish_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->publish_date),
        'public_status' => $i->public_status,
        'message' => $i->message,
        'gender' => $i->gender,
        'avatar' => $i->avatar,
        'showcol' => $showCol,
        'showname' => $showName
      ]);
    }

    return $res;
  }

  public function getPost($slug, Request $r) { // /api/rpc/site/post/get/slug/slug
    $check = $this->objAuth->checkLegit($r->kero_token);
    $valid = $this->objAuth->getPermissions($r->kero_token);

    // Load group colours.
    $ucol = $this->objUser->getGroupColours();

    // Load content.
    if ($valid['blg_editpost']) {
      $get = DB::table('blg_content')
        ->join('users', 'blg_content.user_id', '=', 'users.id')
        ->join('usr_details', 'usr_details.user_id', '=', 'blg_content.user_id')
        ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_content.user_id')
        ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_content.user_id')
        ->where('isPost', 1)
        ->where('slug', $slug)
        ->get(array(
          'blg_content.id',
          'blg_content.user_id',
          'title',
          'slug',
          'post_date',
          'publish_date',
          'public_status',
          'message',
          'username',
          'perm_id',
          'gender',
          'avatar',
          'name_style',
          'display_name'
        ));
    }
    else {
      $get = DB::table('blg_content')
        ->join('users', 'blg_content.user_id', '=', 'users.id')
        ->join('usr_details', 'usr_details.user_id', '=', 'blg_content.user_id')
        ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_content.user_id')
        ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_content.user_id')
        ->where('public_status', 0)
        ->where('isPost', 1)
        ->where('slug', $slug)
        ->get(array(
          'blg_content.id',
          'blg_content.user_id',
          'title',
          'slug',
          'post_date',
          'publish_date',
          'public_status',
          'message',
          'username',
          'perm_id',
          'gender',
          'avatar',
          'name_style',
          'display_name'
        ));
    }

    $res = array();
    setlocale(LC_ALL, 'ja_JP.utf8');

    foreach ($get as $i) {
      $showName = '';
      $showCol = '';
      $showGroupName = '';

      // Display name or username?
      if (!empty($i->display_name)) {
        $showName = $i->display_name;
      }
      else {
        $showName = $i->username;
      }

      // Custom name styling or default?
      if (!empty($i->name_style)) {
        $showCol = $i->name_style;
      }
      else {
        foreach ($ucol as $j) {
          if ($j->id == $i->perm_id) {
            if ($i->gender == 1) $showCol = $j->colour_m;
            else if ($i->gender == 2) $showCol = $j->colour_f;
            else $showCol = $j->colour_u;
          }
        }
      }

      // Group names.
      $gname = $this->objUser->getGroupName($i->user_id);
      $showGroupName = $gname[0]->name;

      array_push($res, [
        'id' => $i->id,
        'user_id' => $i->user_id,
        'title' => $i->title,
        'slug' => $i->slug,
        'post_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->post_date),
        'publish_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->publish_date),
        'public_status' => $i->public_status,
        'message' => $i->message,
        'gender' => $i->gender,
        'avatar' => $i->avatar,
        'showcol' => $showCol,
        'showname' => $showName
      ]);
    }

    return $res;
  }

  public function newPost(Request $r) { // /api/rpc/site/post/new
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_addpost'] == 1) {
        $pubdate = 0;

        if ($r->public_status == 1) {
          if ($r->publish_date <= time()) {
            return \Response::json(array('error' => '投稿日より公開日の方がもっと古いだと不許可です。'));
          }
          else {
            $pubdate = $r->publish_date;
          }
        }
        else {
          $pubdate = time();
        }

        $add = DB::table('blg_content')
          ->insert([
            'user_id' => $check,
            'isPost' => 1,
            'title' => $r->title,
            'slug' => $r->slug,
            'post_date' => time(),
            'publish_date' => $pubdate,
            'sortorder' => 0,
            'public_status' => $r->public_status,
            'isMenu' => 0,
            'message' => $r->message
          ]);

        return \Response::json($add);
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  public function editPost(Request $r) { // /api/rpc/site/post/edit
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_editpost'] == 1) {
        return DB::table('blg_content')
          ->where('id', $r->id)
          ->update([
            'title' => $r->title,
            'slug' => $r->slug,
            'public_status' => $r->public_status,
            'message' => $r->message
          ]);
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  public function deletePost(Request $r) { // /api/rpc/site/post/delete
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_delpost'] == 1) {
        return DB::table('blg_content')->where('id', $r->id)->delete();
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  // Comments
  public function getComments($id) { // /api/rpc/site/comment/get/all/id
    // Load group colours.
    $ucol = $this->objUser->getGroupColours();

    // Load content.
    $get = DB::table('blg_comments')
      ->join('users', 'blg_comments.user_id', '=', 'users.id')
      ->join('usr_details', 'usr_details.user_id', '=', 'blg_comments.user_id')
      ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_comments.user_id')
      ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_comments.user_id')
      ->where('content_id', $id)
      ->orderBy('post_date', 'asc')
      ->get(array(
        'blg_comments.id',
        'blg_comments.user_id',
        'votes',
        'post_date',
        'last_date',
        'isDeleted',
        'message',
        'blg_comments.ip_address',
        'username',
        'perm_id',
        'gender',
        'avatar',
        'name_style',
        'display_name'
      ));

    $res = array();
    $key = 1;
    setlocale(LC_ALL, 'ja_JP.utf8');

    foreach ($get as $i) {
      $showName = '';
      $showCol = '';
      $showGroupName = '';

      // Display name or username?
      if (!empty($i->display_name)) {
        $showName = $i->display_name;
      }
      else {
        $showName = $i->username;
      }

      // Custom name styling or default?
      if (!empty($i->name_style)) {
        $showCol = $i->name_style;
      }
      else {
        foreach ($ucol as $j) {
          if ($j->id == $i->perm_id) {
            if ($i->gender == 1) $showCol = $j->colour_m;
            else if ($i->gender == 2) $showCol = $j->colour_f;
            else $showCol = $j->colour_u;
          }
        }
      }

      // Group names.
      $gname = $this->objUser->getGroupName($i->user_id);
      $showGroupName = $gname[0]->name;

      array_push($res, [
        'key' => $key,
        'id' => $i->id,
        'user_id' => $i->user_id,
        'votes' => $i->votes,
        'post_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->post_date),
        'last_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->last_date),
        'last_unix' => $i->last_date,
        'isDeleted' => $i->isDeleted,
        'message' => $i->message,
        'ip_address' => $i->ip_address,
        'avatar' => $i->avatar,
        'showcol' => $showCol,
        'showname' => $showName
      ]);
      $key++;
    }

    return $res;
  }

  public function getComment($id) { // /api/rpc/site/comment/get/one/id
    // Load group colours.
    $ucol = $this->objUser->getGroupColours();

    // Load content.
    $get = DB::table('blg_comments')
      ->join('users', 'blg_comments.user_id', '=', 'users.id')
      ->join('usr_details', 'usr_details.user_id', '=', 'blg_comments.user_id')
      ->join('usr_profile', 'usr_profile.user_id', '=', 'blg_comments.user_id')
      ->join('usr_perm_id', 'usr_perm_id.user_id', '=', 'blg_comments.user_id')
      ->where('blg_comments.content_id', $id)
      ->orderBy('post_date', 'asc')
      ->get(array(
        'blg_comments.id',
        'blg_comments.user_id',
        'content_id',
        'votes',
        'post_date',
        'last_date',
        'isDeleted',
        'message',
        'blg_comments.ip_address',
        'username',
        'perm_id',
        'gender',
        'avatar',
        'name_style',
        'display_name'
      ));

    $res = array();
    setlocale(LC_ALL, 'ja_JP.utf8');

    foreach ($get as $i) {
      $showName = '';
      $showCol = '';
      $showGroupName = '';

      // Display name or username?
      if (!empty($i->display_name)) {
        $showName = $i->display_name;
      }
      else {
        $showName = $i->username;
      }

      // Custom name styling or default?
      if (!empty($i->name_style)) {
        $showCol = $i->name_style;
      }
      else {
        foreach ($ucol as $j) {
          if ($j->id == $i->perm_id) {
            if ($i->gender == 1) $showCol = $j->colour_m;
            else if ($i->gender == 2) $showCol = $j->colour_f;
            else $showCol = $j->colour_u;
          }
        }
      }

      // Group names.
      $gname = $this->objUser->getGroupName($i->user_id);
      $showGroupName = $gname[0]->name;

      array_push($res, [
        'id' => $i->id,
        'user_id' => $i->user_id,
        'content_id' => $i->content_id,
        'votes' => $i->votes,
        'post_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->post_date),
        'last_date' => strftime('%Y/%m/%d(%a) %H:%M:%S %Z', $i->last_date),
        'last_unix' => $i->last_date,
        'isDeleted' => $i->isDeleted,
        'message' => $i->message,
        'ip_address' => $i->ip_address,
        'avatar' => ($i->avatar ? $i->avatar : 'assets/avatars/haznoavaz.png'),
        'showcol' => $showCol,
        'showname' => $showName
      ]);
    }

    return $res;
  }

  public function newComment(Request $r) { // /api/rpc/site/comment/new
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_addcomment'] == 1) {
        $add = DB::table('blg_comments')
          ->insertGetId([
            'user_id' => $check,
            'content_id' => $r->content_id,
            'votes' => 0,
            'post_date' => time(),
            'last_date' => 0,
            'isDeleted' => 0,
            'message' => $r->message,
            'ip_address' => $r->ip_address
          ]);

        $g = DB::table('blg_content')->select('slug', 'user_id')->where('id', $r->content_id)->first();

        if ($check != $g->user_id) $this->objUser->addNotification($r, $g->user_id, 1, '新規ブログコメント', 'blog/'.$g->slug, 'comment-'.$add);

        return \Response::json($add);
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  public function editComment(Request $r) { // /api/rpc/site/comment/edit
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $owner = DB::table('blg_comments')->select('user_id')->where('id', $r->id)->where('user_id', $r->user)->get()->toArray();
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_editcomment'] == 1) {
        return DB::table('blg_comments')
          ->where('id', $r->id)
          ->update([
            'last_date' => time(),
            'message' => $r->message
          ]);
      }
      else if ($valid['blg_delcomment'] == 1 && $owner[0]->user_id == $check) {
        return DB::table('blg_comments')
          ->where('id', $r->id)
          ->update([
            'last_date' => time(),
            'message' => $r->message
          ]);
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  public function removeComment(Request $r) { // /api/rpc/site/comment/remove
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $owner = DB::table('blg_comments')->select('user_id')->where('id', $r->id)->where('user_id', $r->user)->get()->toArray();
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_delcomment'] == 1) {
        $get = DB::table('blg_comments')
          ->select('isDeleted')
          ->where('id', $r->id)
          ->get();

        $mod = 0;

        foreach ($get as $i) {
          if ($i->isDeleted == 1) $mod = 0;
          else $mod = 1;
        }

        return DB::table('blg_comments')
          ->where('id', $r->id)
          ->update([
            'isDeleted' => $mod
          ]);
      }
      else if ($valid['blg_delowncomment'] == 1 && $owner[0]->user_id == $check) {
        $get = DB::table('blg_comments')
          ->select('isDeleted')
          ->where('id', $r->id)
          ->get();

        foreach ($get as $i) {
          if ($i->isDeleted == 1) return "不許可";
        }

        return DB::table('blg_comments')
          ->where('id', $r->id)
          ->update([
            'isDeleted' => 1
          ]);
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  public function voteComment(Request $r) { // /api/rpc/site/comment/vote
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_addcomment'] == 1) {
        $get = DB::table('blg_comments')
          ->select('votes')
          ->where('id', $r->id)
          ->get();

        $mod = 0;

        foreach ($get as $i) {
          $mod = $i->votes;
        }

        return DB::table('blg_comments')
          ->where('id', $r->id)
          ->update([
            'votes' => $r->votemod
          ]);
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  // Pages
  public function getPagesInMenu() { // /api/rpc/site/page/get/menu
    $get = DB::table('blg_content')
      ->select('title', 'slug')
      ->where('public_status', 0)
      ->where('isPost', 0)
      ->where('isMenu', 1)
      ->orderBy('sortorder', 'asc')
      ->get();

    $res = array();
    $key = 0;

    foreach ($get as $i) {
      array_push($res, [
        'key' => $key,
        'title' => $i->title,
        'slug' => $i->slug
      ]);
      $key++;
    }

    return $res;
  }

  public function getPages() { // /api/rpc/site/page/all
    $get = DB::table('blg_content')
      ->select('id', 'title', 'slug', 'public_status', 'isMenu', 'message')
      ->where('public_status', 0)
      ->where('isPost', 0)
      ->orderBy('sortorder', 'asc')
      ->get();

    $res = array();
    $key = 0;

    foreach ($get as $i) {
      array_push($res, [
        'key' => $key,
        'id' => $i->id,
        'title' => $i->title,
        'slug' => $i->slug,
        'public_status' => $i->public_status,
        'isMenu' => $i->isMenu,
        'message' => $i->message
      ]);
      $key++;
    }

    return $res;
  }

  public function getPageSlug($id) { // /api/rpc/site/page/get/id/id
    return DB::table('blg_content')
      ->select('slug')
      ->where('public_status', 0)
      ->where('isPost', 0)
      ->where('id', $id)
      ->get();
  }

  public function getPage2 ($slug, $kero) {
    $valid = $this->objAuth->getPermissions($kero);

    if ($valid['blg_editpage'] == 1) {
      return DB::table('blg_content')
        ->select('id', 'title', 'slug', 'isMenu', 'public_status', 'message')
        ->where('isPost', 0)
        ->where('slug', $slug)
        ->orderBy('sortorder', 'asc')
        ->first();
    }
    else {
      return DB::table('blg_content')
        ->select('id', 'title', 'slug', 'isMenu', 'public_status', 'message')
        ->where('public_status', 0)
        ->where('isPost', 0)
        ->where('slug', $slug)
        ->orderBy('sortorder', 'asc')
        ->first();
    }
  }

  public function getPage($slug, Request $r) { // /api/rpc/site/page/get/slug/slug
    $valid = $this->objAuth->getPermissions($r->kero_token);

    if ($valid['blg_editpage'] == 1) {
      return DB::table('blg_content')
        ->select('id', 'title', 'slug', 'isMenu', 'public_status', 'message')
        ->where('isPost', 0)
        ->where('slug', $slug)
        ->orderBy('sortorder', 'asc')
        ->get();
    }
    else {
      return DB::table('blg_content')
        ->select('id', 'title', 'slug', 'message')
        ->where('public_status', 0)
        ->where('isPost', 0)
        ->where('slug', $slug)
        ->orderBy('sortorder', 'asc')
        ->get();
    }
  }

  public function newPage(Request $r) { // /api/rpc/site/page/new
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_addpage'] == 1) {
        $lorder = DB::table('blg_content')
          ->select('sortorder')
          ->where('isPost', 0)
          ->orderBy('sortorder', 'desc')
          ->first()->sortorder;

        $add = DB::table('blg_content')
          ->insert([
            'user_id' => 0,
            'isPost' => 0,
            'title' => $r->title,
            'slug' => $r->slug,
            'post_date' => 0,
            'publish_date' => 0,
            'sortorder' => ($r->isMenu == 1 ? $lorder : 0),
            'public_status' => $r->public_status,
            'isMenu' => ($r->isMenu ? 1 : 0),
            'message' => $r->message
          ]);

        return \Response::json($add);
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  public function editPage(Request $r) { // /api/rpc/site/page/edit
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_editpage'] == 1) {
        return DB::table('blg_content')
          ->where('id', $r->id)
          ->update([
            'title' => $r->title,
            'slug' => $r->slug,
            'public_status' => $r->public_status,
            'isMenu' => $r->isMenu,
            'message' => $r->message
          ]);
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  public function deletePage(Request $r) { // /api/rpc/site/page/delete
    $check = $this->objAuth->checkLegit($r->kero_token);

    if ($check == 0) {
      return \Response::json(array('error' => 'ログインされませんでした。'));
    }
    else {
      $valid = $this->objAuth->getPermissions($r->kero_token);

      if ($valid['blg_delpage'] == 1) {
        return DB::table('blg_content')->where('id', $r->id)->delete();
      }
      else {
        return \Response::json(array('error' => '不許可。'));
      }
    }
  }

  // Contact Fields
  public function getContactFields() { // /api/rpc/site/contact/getcontactfields
    return false;
  }

  public function getContactField($id) { // /api/rpc/site/contact/getcontactfield/id
    return false;
  }

  // Portfolio Categories
  public function getPortfolioCategories() { // /api/rpc/site/portfolio/getportfoliocategories
    return false;
  }

  public function getPortfolioCategory($id) { // /api/rpc/site/portfolio/getportfoliocategory/id
    return false;
  }

  public function newPortfolioCategory(Request $r) { // /api/rpc/site/portfolio/newcategory
    return false;
  }

  public function editPortfolioCategory(Request $r) { // /api/rpc/site/portfolio/editcategory
    return false;
  }

  public function removePortfolioCategory(Request $r) { // /api/rpc/site/portfolio/removecategory
    return false;
  }

  public function deletePortfolioCategory(Request $r) { // /api/rpc/site/portfolio/deletecategory
    return false;
  }

  // Portfolio Items
  public function getPortfolioItems($id) { // /api/rpc/site/portfolio/getportfolioitems/id
    return false;
  }

  public function getPortfolioItem($id) { // /api/rpc/site/portfolio/getportfolioitems/id
    return false;
  }

  public function newPortfolioItem(Request $r) { // /api/rpc/site/portfolio/newitem
    return false;
  }

  public function editPortfolioItem(Request $r) { // /api/rpc/site/portfolio/edititem
    return false;
  }

  public function removePortfolioItem(Request $r) { // /api/rpc/site/portfolio/removeitem
    return false;
  }

  public function deletePortfolioItem(Request $r) { // /api/rpc/site/portfolio/deleteitem
    return false;
  }

  // Admin
  public function getSettings() { // /api/rpc/site/admin/getsettings
    return DB::table('blg_settings')
      ->select('sitename', 'description', 'tags', 'blog_status', 'homepage', 'announcement', 'theme', 'locale')
      ->get();
  }

  public function setSettings(Request $r) { // /api/rpc/site/admin/setsettings
    return DB::table('blg_settings')
      ->update([
        'sitename' => $r->sitename,
        'description' => $r->description,
        'tags' => $r->tags,
        'blog_status' => $r->blog_status,
        'homepage' => $r->homepage,
        'announcement' => $r->announcement,
        'theme' => $r->theme,
        'locale' => $r->locale
      ]);
  }
}
